# This Build.PL for Dist-Zilla-PluginBundle-Author-IOANR was generated by Dist::Zilla::Plugin::ModuleBuildTiny 0.015.
use strict;
use warnings;

use v5.24.0;
use Module::Build::Tiny 0.034;
Build_PL();
